#include "project-defs.h"
#include <stdbool.h>
#define _COMM_LINK_I2C_C
#include "comm-link-i2c.h"

static void initialise(const void *linkConfig) {
	// L'initialisation de l'interface I2C a déjà été faite dans main.c.
}

static void write(const void *linkConfig, CommLinkDataType dataType, const uint8_t *data, int count) {
	while (I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY) != RESET);

	I2C_GenerateSTART(I2C1, ENABLE);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

	I2C_Send7bitAddress(I2C1, ((const CommLinkI2CConfig *) linkConfig)->i2cAddress << 1, I2C_Direction_Transmitter);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

	// On envoie le "control byte" avec Co = 0 et D/C# = dataType
	// (voir data sheet SH1106 page 14 pour le format de message).
	I2C_SendData(I2C1, dataType << 6);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

	// On envoie les octets de données.
	for (int i = 0; i < count; i++) {
		I2C_SendData(I2C1, data[i]);
		while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	}

	I2C_GenerateSTOP(I2C1, ENABLE);
}

static void read(const void *linkConfig, CommLinkDataType dataType, uint8_t *buffer, int count) {
	while (I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY) != RESET);

	I2C_GenerateSTART(I2C1, ENABLE);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

	I2C_Send7bitAddress(I2C1, ((const CommLinkI2CConfig *) linkConfig)->i2cAddress << 1, I2C_Direction_Transmitter);
	while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));

	int i = 0;

	// On reçoit normalement (c'est-à-dire avec un ACK) les (count - 1) premiers octets.
	for (; i < (count - 1); i++) {
		// On attend que la réponse soit arrivée.
		while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);
		// et on stocke l'octet reçu.
		buffer[i] = I2C_ReceiveData(I2C1);
	}

	// On attend que le dernier octet soit arrivé.
	while (I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);
	// On envoie un NAK (voir page 14 de la data sheet du SH1106).
	I2C_AcknowledgeConfig(I2C1, DISABLE);
	// On stocke l'octet reçu.
	buffer[i] = I2C_ReceiveData(I2C1);
	// On reconfigure un ACK par défaut pour les transactions suivantes.
	I2C_AcknowledgeConfig(I2C1, ENABLE);

	I2C_GenerateSTOP(I2C1, ENABLE);
}

static bool hasReset(const void *linkConfig) {
	// Non applicable
	return false;
}

static void assertReset(const void *linkConfig) {
	// Non applicable
}

static void deassertReset(const void *linkConfig) {
	// Non applicable
}

static void rawWrite(const void *linkConfig, CommLinkDataType dataType, const uint8_t *data, int count) {
	write(linkConfig, dataType, data, count);
}

static int getBusWidth(const void *linkConfig) {
	return 8;
}

const CommLinkImpl COMM_LINK_I2C_IMPL = {
	.initialise = initialise,
	.write = write,
	.read = read,
	.hasReset = hasReset,
	.assertReset = assertReset,
	.deassertReset = deassertReset,
	.rawWrite = rawWrite,
	.getBusWidth = getBusWidth,
};
